@@ -57,41 +57,6 @@ OUTER_CONFIGURATIONS_SCHEMA = {
     "description": "Instance group application configurations."
 }
 
-ONDEMAND_CAPACITY_RESERVATION_OPTIONS_SCHEMA = {
-    "type": "object",
-    "properties" : {
-        "UsageStrategy": {
-            "type": "string",
-            "description": "The strategy of whether to use available capacity reservations to fulfill On-Demand capacity.",
-            "enum": ["use-capacity-reservations-first"]
-        },
-        "CapacityReservationPreference": {
-            "type": "string",
-            "description": "The preference of the capacity reservation of the instance.",
-            "enum": [
-                "open",
-                "none"
-            ]
-        },
-        "CapacityReservationResourceGroupArn": {
-            "type": "string",
-            "description": "The ARN of the capacity reservation resource group in which to run the instance."
-        }
-    }
-}
-
-SPOT_ALLOCATION_STRATEGY_SCHEMA = {
-    "type": "string",
-    "description": "The strategy to use to launch Spot instance fleets.",
-    "enum": ["capacity-optimized", "price-capacity-optimized", "lowest-price", "diversified", "capacity-optimized-prioritized"]
-}
-
-ONDEMAND_ALLOCATION_STRATEGY_SCHEMA = {
-    "type": "string",
-    "description": "The strategy to use to launch On-Demand instance fleets.",
-    "enum": ["lowest-price", "prioritized"]
-}
-
 INSTANCE_GROUPS_SCHEMA = {
     "type": "array",
     "items": {
@@ -378,11 +343,6 @@ INSTANCE_FLEETS_SCHEMA = {
                             "type": "string",
                             "description": "The AMI ID of a custom AMI to use when Amazon EMR provisions EC2 instances."
                         },
-                        "Priority": {
-                            "type": "double",
-                            "description": "The priority at which Amazon EMR launches the EC2 instances with this instance type. "
-                                "Priority starts at 0, which is the highest priority. Amazon EMR considers the highest priority first."
-                        },
                         "EbsConfiguration": {
                             "type": "object",
                             "description": "EBS configuration that is associated with the instance group.",
@@ -446,8 +406,33 @@ INSTANCE_FLEETS_SCHEMA = {
                     "OnDemandSpecification": {
                         "type": "object",
                         "properties": {
-                            "AllocationStrategy": ONDEMAND_ALLOCATION_STRATEGY_SCHEMA,
-                            "CapacityReservationOptions": ONDEMAND_CAPACITY_RESERVATION_OPTIONS_SCHEMA
+                            "AllocationStrategy": {
+                                "type": "string",
+                                "description": "The strategy to use in launching On-Demand instance fleets.",
+                                "enum": ["lowest-price"]
+                            },
+                            "CapacityReservationOptions": {
+                                "type": "object",
+                                "properties" : {
+                                    "UsageStrategy": {
+                                        "type": "string",
+                                        "description": "The strategy of whether to use unused Capacity Reservations for fulfilling On-Demand capacity.",
+                                        "enum": ["use-capacity-reservations-first"]
+                                    },
+                                    "CapacityReservationPreference": {
+                                        "type": "string",
+                                        "description": "The preference of the instance's Capacity Reservation.",
+                                        "enum": [
+                                            "open",
+                                            "none"
+                                        ]
+                                    },
+                                    "CapacityReservationResourceGroupArn": {
+                                        "type": "string",
+                                        "description": "The ARN of the Capacity Reservation resource group in which to run the instance."
+                                    }
+                                }
+                            }
                         }
                     },
                     "SpotSpecification": {
@@ -469,7 +454,11 @@ INSTANCE_FLEETS_SCHEMA = {
                                 "type": "integer",
                                 "description": "Block duration in minutes."
                             },
-                            "AllocationStrategy": SPOT_ALLOCATION_STRATEGY_SCHEMA
+                            "AllocationStrategy": {
+                                "type": "string",
+                                "description": "The strategy to use in launching Spot instance fleets.",
+                                "enum": ["capacity-optimized"]
+                            }
                         }
                     }
                 }
@@ -483,8 +472,7 @@ INSTANCE_FLEETS_SCHEMA = {
                             "TimeoutDurationMinutes": {
                                 "type" : "integer",
                                 "description": "The time, in minutes, after which the resize will be stopped if requested resources are unavailable."
-                            },
-                            "AllocationStrategy": SPOT_ALLOCATION_STRATEGY_SCHEMA
+                            }
                         }
                     },
                     "OnDemandResizeSpecification": {
@@ -493,9 +481,7 @@ INSTANCE_FLEETS_SCHEMA = {
                             "TimeoutDurationMinutes": {
                                 "type" : "integer",
                                 "description": "The time, in minutes, after which the resize will be stopped if requested resources are unavailable."
-                            },
-                            "AllocationStrategy": ONDEMAND_ALLOCATION_STRATEGY_SCHEMA,
-                            "CapacityReservationOptions": ONDEMAND_CAPACITY_RESERVATION_OPTIONS_SCHEMA
+                            }
                         }
                     }
                 }
